==> GotoBLAS-1.00.tar.gz: GotoBLAS/level3/trsm/trsm_R.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "common.h"

const static FLOAT dm1 = -1.;

#ifdef CONJ
#define GEMM_KERNEL   GEMM_KERNEL_R
#if (!defined(TRANSA) && defined(UPPER)) || (defined(TRANSA) && !defined(UPPER))
#define TRSM_KERNEL   TRSM_KERNEL_RR
#else
#define TRSM_KERNEL   TRSM_KERNEL_RC
#endif
#else
#define GEMM_KERNEL   GEMM_KERNEL_N
#if (!defined(TRANSA) && defined(UPPER)) || (defined(TRANSA) && !defined(UPPER))
#define TRSM_KERNEL   TRSM_KERNEL_RN
#else
#define TRSM_KERNEL   TRSM_KERNEL_RT
#endif
#endif

int CNAME(long m, long n, long dummy1,
	  FLOAT dummyalpha1, 
#ifdef COMPLEX
	  FLOAT dummyalpha2,
#endif
	  FLOAT *a, long lda,
	  FLOAT *dummy3, long dummy4, 
	  FLOAT *b, long ldb, FLOAT *buffer){

  long ls, is, js;
  long min_l, min_i, min_j;
  long offset;
#if (defined(TRANSA) && defined(UPPER)) || (!defined(TRANSA) && !defined(UPPER))
  int start_ls, start_js;
#endif

#ifndef PPC440
  FLOAT *sa, *sb;
  sa = (FLOAT *)((void *)buffer + GEMM_BUFFER_A_OFFSET);
  sb = (FLOAT *)((void *)buffer + GEMM_BUFFER_B_OFFSET
		 + ((GEMM_P * GEMM_Q * COMPSIZE * SIZE + GEMM_ALIGN) & ~GEMM_ALIGN));
#else
  extern FLOAT *sa, *sb;
#endif

#if (defined(TRANSA) && !defined(UPPER)) || (!defined(TRANSA) && defined(UPPER))
  for(ls = 0; ls < n; ls += GEMM_Q){
    min_l = n - ls;
    if (min_l > GEMM_Q) min_l = GEMM_Q;
    
    for(js = 0; js < n; js += GEMM_R){
      min_j = n - js;
      if (min_j > GEMM_R) min_j = GEMM_R;
#else
  start_js = 0;
  while (start_js < n) start_js += GEMM_R;
  start_js -= GEMM_R;

  start_ls = 0;
  while (start_ls < n) start_ls += GEMM_Q;
  start_ls -= GEMM_Q;

  for(ls = start_ls; ls >= 0; ls -= GEMM_Q){
    min_l = n - ls;
    if (min_l > GEMM_Q) min_l = GEMM_Q;
    
    for(js = start_js; js >= 0; js -= GEMM_R){
      min_j = n - js;
      if (min_j > GEMM_R) min_j = GEMM_R;
#endif

      
#if (defined(TRANSA) && !defined(UPPER)) || (!defined(TRANSA) && defined(UPPER))
      
      if (js + GEMM_R >= ls) {
#ifndef TRANSA
	TRSM_OUNCOPY(min_l, min_j, a + (ls + js * lda) * COMPSIZE, lda, js - ls, sb);
#else
	TRSM_OLTCOPY(min_l, min_j, a + (js + ls * lda) * COMPSIZE, lda, js - ls, sb);
#endif

	for(is = 0; is < m; is += GEMM_P){
	  min_i = m - is;
	  if (min_i > GEMM_P) min_i = GEMM_P;
	  
	  GEMM_ITCOPY(min_l, min_i, b + (is + ls * ldb) * COMPSIZE, ldb, sa);

	  if (ls + GEMM_Q >= js) {
	    offset = ls - js;
	    if (offset >= 0){
	      TRSM_KERNEL(min_i, MIN(min_l, min_j - offset), min_l, dm1,
#ifdef COMPLEX
			  ZERO,
#endif
			  sa,
			  sb + min_l * offset * COMPSIZE, 
			  b + (is + (js + offset) * ldb) * COMPSIZE, ldb, 0);
	    } else {
	      TRSM_KERNEL(min_i, MIN(min_l + offset, min_j), min_l, dm1,
#ifdef COMPLEX
			  ZERO,
#endif
			  sa,
			  sb, 
			  b + (is + js * ldb) * COMPSIZE, ldb, offset);
	    }

	    if (min_j > min_l + offset) {
	      GEMM_KERNEL(min_i, min_j - min_l - offset, min_l, dm1, 
#ifdef COMPLEX
			    ZERO,
#endif
			    sa, 
			    sb + min_l * (min_l + offset) * COMPSIZE,
			    b + (is + (js + min_l + offset) * ldb) * COMPSIZE, ldb); 
	    }
	    
	  } else {
	    GEMM_KERNEL(min_i, min_j, min_l, dm1, 
#ifdef COMPLEX
			  ZERO,
#endif
			  sa, sb, b + (is + js * ldb) * COMPSIZE, ldb); 
	  }
	}
      }

#else

      if (ls + GEMM_Q >= js) {
#ifndef TRANSA
	TRSM_OLNCOPY(min_l, min_j, a + (ls + js * lda) * COMPSIZE, lda, js - ls, sb);
#else
	TRSM_OUTCOPY(min_l, min_j, a + (js + ls * lda) * COMPSIZE, lda, js - ls, sb);
#endif

	for(is = 0; is < m; is += GEMM_P){
	  min_i = m - is;
	  if (min_i > GEMM_P) min_i = GEMM_P;
	  
	  GEMM_ITCOPY(min_l, min_i, b + (is + ls * ldb) * COMPSIZE, ldb, sa);

	  if (js + GEMM_R >= ls) {
	    offset = ls - js;

	    if (offset >= 0){

	      TRSM_KERNEL(min_i, MIN(min_l, min_j - offset), min_l, dm1,
#ifdef COMPLEX
			  ZERO,
#endif
			  sa,
			  sb + min_l * offset * COMPSIZE, 
			  b + (is + (js + offset) * ldb) * COMPSIZE, ldb, 0);

	      if (offset > 0) {
		GEMM_KERNEL(min_i, offset, min_l, dm1, 
#ifdef COMPLEX
			      ZERO,
#endif
			      sa, 
			      sb,
			      b + (is + js * ldb) * COMPSIZE, ldb); 
	      }

	    } else {
	      TRSM_KERNEL(min_i, MIN(min_l + offset, min_j), min_l, dm1,
#ifdef COMPLEX
			  ZERO,
#endif
			  sa,
			  sb, 
			  b + (is + js * ldb) * COMPSIZE, ldb, offset);
	    }
	  } else {
	    GEMM_KERNEL(min_i, min_j, min_l, dm1, 
#ifdef COMPLEX
			  ZERO,
#endif
			  sa, sb, b + (is + js * ldb) * COMPSIZE, ldb); 
	  }
	}
      }
#endif

      } /* end of ls */
     } /* end of js */
      
  return 0;
}

==> GotoBLAS-1.10.tar.gz: GotoBLAS/level3/trsm/trsm_R.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "common.h"

const static FLOAT dm1 = -1.;

#ifdef CONJ
#define GEMM_KERNEL   GEMM_KERNEL_R
#if (!defined(TRANSA) && defined(UPPER)) || (defined(TRANSA) && !defined(UPPER))
#define TRSM_KERNEL   TRSM_KERNEL_RR
#else
#define TRSM_KERNEL   TRSM_KERNEL_RC
#endif
#else
#define GEMM_KERNEL   GEMM_KERNEL_N
#if (!defined(TRANSA) && defined(UPPER)) || (defined(TRANSA) && !defined(UPPER))
#define TRSM_KERNEL   TRSM_KERNEL_RN
#else
#define TRSM_KERNEL   TRSM_KERNEL_RT
#endif
#endif

int CNAME(long m, long n, long dummy1,
	  FLOAT dummyalpha1, 
#ifdef COMPLEX
	  FLOAT dummyalpha2,
#endif
	  FLOAT *a, long lda,
	  FLOAT *dummy3, long dummy4, 
	  FLOAT *b, long ldb, FLOAT *buffer){

  long ls, is, js;
  long min_l, min_i, min_j;
  long offset;
#if (defined(TRANSA) && defined(UPPER)) || (!defined(TRANSA) && !defined(UPPER))
  int start_ls, start_js;
#endif

#ifndef PPC440
  FLOAT *sa, *sb;
  sa = (FLOAT *)((long)buffer + GEMM_BUFFER_A_OFFSET);
  sb = (FLOAT *)((long)buffer + GEMM_BUFFER_B_OFFSET
		 + ((GEMM_P * GEMM_Q * COMPSIZE * SIZE + GEMM_BUFFER_A_OFFSET * COMPSIZE + GEMM_ALIGN) & ~GEMM_ALIGN));
#else
  extern FLOAT *sa, *sb;
#endif

#if (defined(TRANSA) && !defined(UPPER)) || (!defined(TRANSA) && defined(UPPER))
  for(ls = 0; ls < n; ls += GEMM_Q){
    min_l = n - ls;
    if (min_l > GEMM_Q) min_l = GEMM_Q;
    
    for(js = 0; js < n; js += GEMM_R){
      min_j = n - js;
      if (min_j > GEMM_R) min_j = GEMM_R;
#else
  start_js = 0;
  while (start_js < n) start_js += GEMM_R;
  start_js -= GEMM_R;

  start_ls = 0;
  while (start_ls < n) start_ls += GEMM_Q;
  start_ls -= GEMM_Q;

  for(ls = start_ls; ls >= 0; ls -= GEMM_Q){
    min_l = n - ls;
    if (min_l > GEMM_Q) min_l = GEMM_Q;
    
    for(js = start_js; js >= 0; js -= GEMM_R){
      min_j = n - js;
      if (min_j > GEMM_R) min_j = GEMM_R;
#endif

      
#if (defined(TRANSA) && !defined(UPPER)) || (!defined(TRANSA) && defined(UPPER))
      
      if (js + GEMM_R >= ls) {
#ifndef TRANSA
	TRSM_OUNCOPY(min_l, min_j, a + (ls + js * lda) * COMPSIZE, lda, js - ls, sb);
#else
	TRSM_OLTCOPY(min_l, min_j, a + (js + ls * lda) * COMPSIZE, lda, js - ls, sb);
#endif

	for(is = 0; is < m; is += GEMM_P){
	  min_i = m - is;
	  if (min_i > GEMM_P) min_i = GEMM_P;
	  
	  GEMM_ITCOPY(min_l, min_i, b + (is + ls * ldb) * COMPSIZE, ldb, sa);

	  if (ls + GEMM_Q >= js) {
	    offset = ls - js;
	    if (offset >= 0){
	      TRSM_KERNEL(min_i, MIN(min_l, min_j - offset), min_l, dm1,
#ifdef COMPLEX
			  ZERO,
#endif
			  sa,
			  sb + min_l * offset * COMPSIZE, 
			  b + (is + (js + offset) * ldb) * COMPSIZE, ldb, 0);
	    } else {
	      TRSM_KERNEL(min_i, MIN(min_l + offset, min_j), min_l, dm1,
#ifdef COMPLEX
			  ZERO,
#endif
			  sa,
			  sb, 
			  b + (is + js * ldb) * COMPSIZE, ldb, offset);
	    }

	    if (min_j > min_l + offset) {
	      GEMM_KERNEL(min_i, min_j - min_l - offset, min_l, dm1, 
#ifdef COMPLEX
			    ZERO,
#endif
			    sa, 
			    sb + min_l * (min_l + offset) * COMPSIZE,
			    b + (is + (js + min_l + offset) * ldb) * COMPSIZE, ldb); 
	    }
	    
	  } else {
	    GEMM_KERNEL(min_i, min_j, min_l, dm1, 
#ifdef COMPLEX
			  ZERO,
#endif
			  sa, sb, b + (is + js * ldb) * COMPSIZE, ldb); 
	  }
	}
      }

#else

      if (ls + GEMM_Q >= js) {
#ifndef TRANSA
	TRSM_OLNCOPY(min_l, min_j, a + (ls + js * lda) * COMPSIZE, lda, js - ls, sb);
#else
	TRSM_OUTCOPY(min_l, min_j, a + (js + ls * lda) * COMPSIZE, lda, js - ls, sb);
#endif

	for(is = 0; is < m; is += GEMM_P){
	  min_i = m - is;
	  if (min_i > GEMM_P) min_i = GEMM_P;
	  
	  GEMM_ITCOPY(min_l, min_i, b + (is + ls * ldb) * COMPSIZE, ldb, sa);

	  if (js + GEMM_R >= ls) {
	    offset = ls - js;

	    if (offset >= 0){

	      TRSM_KERNEL(min_i, MIN(min_l, min_j - offset), min_l, dm1,
#ifdef COMPLEX
			  ZERO,
#endif
			  sa,
			  sb + min_l * offset * COMPSIZE, 
			  b + (is + (js + offset) * ldb) * COMPSIZE, ldb, 0);

	      if (offset > 0) {
		GEMM_KERNEL(min_i, offset, min_l, dm1, 
#ifdef COMPLEX
			      ZERO,
#endif
			      sa, 
			      sb,
			      b + (is + js * ldb) * COMPSIZE, ldb); 
	      }

	    } else {
	      TRSM_KERNEL(min_i, MIN(min_l + offset, min_j), min_l, dm1,
#ifdef COMPLEX
			  ZERO,
#endif
			  sa,
			  sb, 
			  b + (is + js * ldb) * COMPSIZE, ldb, offset);
	    }
	  } else {
	    GEMM_KERNEL(min_i, min_j, min_l, dm1, 
#ifdef COMPLEX
			  ZERO,
#endif
			  sa, sb, b + (is + js * ldb) * COMPSIZE, ldb); 
	  }
	}
      }
#endif

      } /* end of ls */
     } /* end of js */
      
  return 0;
}
